// importing necessary modules
import {
  csv,
  select,
  pie,
  arc,
  scaleOrdinal,
  format,
} from 'd3';

// declaring constants by reading svg tag from index.html
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');
const xAxisTickFormat = (number) => format('.4~g')(number);

// Start rendering pie chart to html
const render = (data, total) => {
  const pieData = pie().value(
    (d) =>
      d.SalesNorthAmerica + d.SalesEurope + d.SalesJapan,
  )(data);

  // Colors for in Pie Chart
  const colors = scaleOrdinal()
    .domain(data)
    .range([
      '#FFD700',
      '#018670',
      '#B22222',
      '#8dc63f',
      '#f58026',
      '#4b4b4b',
      '#9acbc2',
      '#00b5cc',
      '#014140',
      '#d9d5d5',
      '#9400D3',
    ]);

  const segments = arc()
    .innerRadius(0)
    .outerRadius(170)
    .padAngle(0)
    .padRadius(0);

  // Rendering section of pie chart
  const sections = svg
    .append('g')
    .attr('transform', `translate(250,250)`)
    .selectAll('path')
    .data(pieData);

  // Rendering color to the section to pie chart
  sections
    .enter()
    .append('path')
    .attr('d', segments)
    .attr('fill', (d) => colors(d.data.Title));

  // Redering legends for pie chart
  const legends = svg
    .append('g')
    .attr('transform', 'translate(500,100)')
    .selectAll('.legends')
    .data(pieData);

  const legend = legends
    .enter()
    .append('g')
    .classed('.legends', true)
    .attr('transform', (d, i) => {
      return `translate(0,${(i + 1) * 25})`;
    });

  //redering legend colors to pie chart
  legend
    .append('rect')
    .attr('width', 20)
    .attr('height', 20)
    .attr('fill', (d) => colors(d.data.Title));

  //Redering legends brands text
  legend
    .append('text')
    .attr('x', 25)
    .attr('y', 15)
    .attr('class', 'legend_text')
    .text((d) => d.data.Title);

  //Redering legends brands sales value percentage text
  legend
    .append('text')
    .attr('x', 300)
    .attr('y', 15)
    .attr('class', 'legend_value')
    .text(
      (d) =>
        xAxisTickFormat(
          ((d.data.SalesNorthAmerica +
            d.data.SalesEurope +
            d.data.SalesJapan) /
            total) *
            100,
        ) + '%',
    );

  //Redering legends brands sales value text
  legend
    .append('text')
    .attr('x', 350)
    .attr('y', 15)
    .attr('class', 'legend_value')
    .text((d) =>
      xAxisTickFormat(
        d.data.SalesNorthAmerica +
          d.data.SalesEurope +
          d.data.SalesJapan,
      ),
    );
};

// load data from csv file link
const run = async () => {
  const data = await csv(
    'https://gist.githubusercontent.com/Satishnarne/c7851ee5fa12a262ee386489e5f550e9/raw/fe17ae77e556caa44f851d8e7529065fb241a37a/ICE05-DATA.csv',
  );

  let total = 0;
  data.forEach((d) => {
    d.SalesNorthAmerica = +d.SalesNorthAmerica;
    d.SalesEurope = +d.SalesEurope;
    d.SalesJapan = +d.SalesJapan;
    total +=
      d.SalesNorthAmerica + d.SalesEurope + d.SalesJapan;
  });

  render(data, total);
};

//Initial Function call to render pie chart
run();
